import { ContextType, Controller, HttpServer, PipeTransform, Type } from '@nestjs/common/interfaces';
import { GuardsConsumer } from '@nestjs/core/guards/guards-consumer';
import { GuardsContextCreator } from '@nestjs/core/guards/guards-context-creator';
import { InterceptorsConsumer } from '@nestjs/core/interceptors/interceptors-consumer';
import { InterceptorsContextCreator } from '@nestjs/core/interceptors/interceptors-context-creator';
import { PipesConsumer } from '@nestjs/core/pipes/pipes-consumer';
import { PipesContextCreator } from '@nestjs/core/pipes/pipes-context-creator';
import { IRouteParamsFactory } from '@nestjs/core/router/interfaces/route-params-factory.interface';
import { HandlerMetadata } from '@nestjs/core/helpers/handler-metadata-storage';
import { RouteParamMetadata } from '@nestjs/common';
import { ParamProperties } from '@nestjs/core/router/router-execution-context';
import { Fn } from '../types';
import { ExecutionContextHost } from '@nestjs/core/helpers/execution-context-host';
import { IRpcHandler } from '../interfaces';
import { CustomHeader } from '@nestjs/core/router/router-response-controller';
export declare class JsonRpcContextCreator {
    private readonly paramsFactory;
    private readonly pipesContextCreator;
    private readonly pipesConsumer;
    private readonly guardsContextCreator;
    private readonly guardsConsumer;
    private readonly interceptorsContextCreator;
    private readonly interceptorsConsumer;
    private readonly applicationRef;
    private readonly handlerMetadataStorage;
    private readonly contextUtils;
    private readonly responseController;
    private readonly DEFAULT_RESPONSE_CODE;
    constructor(paramsFactory: IRouteParamsFactory, pipesContextCreator: PipesContextCreator, pipesConsumer: PipesConsumer, guardsContextCreator: GuardsContextCreator, guardsConsumer: GuardsConsumer, interceptorsContextCreator: InterceptorsContextCreator, interceptorsConsumer: InterceptorsConsumer, applicationRef: HttpServer);
    create<TContext extends string = ContextType>(instance: IRpcHandler<any> | Type<any>, callback: (...args: unknown[]) => unknown, methodName: string, module: string, contextId?: import("@nestjs/core").ContextId, inquirerId?: string): <TRequest, TResponse>(req: TRequest, res: TResponse, next: Fn) => Promise<any>;
    getMetadata<TMetadata, TContext extends string = ContextType>(instance: Controller, callback: (...args: unknown[]) => unknown, methodName: string, contextType: TContext): HandlerMetadata;
    exchangeKeysForValues(keys: string[], metadata: RouteParamMetadata, moduleContext: string, contextId?: import("@nestjs/core").ContextId, inquirerId?: string, contextFactory?: (args: unknown[]) => ExecutionContextHost): ParamProperties[];
    getCustomFactory(factory: (...args: unknown[]) => void, data: unknown, contextFactory: (args: unknown[]) => ExecutionContextHost): (...args: unknown[]) => unknown;
    getParamValue<T>(value: T, { metatype, type, data, }: {
        metatype: Type<any> | undefined;
        type: any;
        data: any;
    }, pipes: PipeTransform[]): Promise<unknown>;
    createGuardsFn<TContext extends string = ContextType>(guards: any[], instance: Controller, callback: (...args: any[]) => any, contextType?: TContext): Fn | null;
    createPipesFn(pipes: PipeTransform[], paramsOptions: (ParamProperties & {
        metatype?: unknown;
    })[]): <TRequest, TResponse>(args: unknown[], req: TRequest, res: TResponse, next: Fn) => Promise<void>;
    createHandleResponseFn(): <TResult, TResponse>(result: TResult, res: TResponse) => Promise<TResult>;
    transformToResult(resultOrDeffered: any): Promise<any>;
    reflectResponseHeaders(callback: (...args: unknown[]) => unknown): CustomHeader[];
}
