"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.JsonRpcContextCreator = void 0;
const handler_metadata_storage_1 = require("@nestjs/core/helpers/handler-metadata-storage");
const context_utils_1 = require("@nestjs/core/helpers/context-utils");
const common_1 = require("@nestjs/common");
const constants_1 = require("@nestjs/core/injector/constants");
const constants_2 = require("@nestjs/common/constants");
const route_paramtypes_enum_1 = require("@nestjs/common/enums/route-paramtypes.enum");
const shared_utils_1 = require("@nestjs/common/utils/shared.utils");
const constants_3 = require("@nestjs/core/guards/constants");
const json_rpc_response_controller_1 = require("./json-rpc-response-controller");
class JsonRpcContextCreator {
    constructor(paramsFactory, pipesContextCreator, pipesConsumer, guardsContextCreator, guardsConsumer, interceptorsContextCreator, interceptorsConsumer, applicationRef) {
        this.paramsFactory = paramsFactory;
        this.pipesContextCreator = pipesContextCreator;
        this.pipesConsumer = pipesConsumer;
        this.guardsContextCreator = guardsContextCreator;
        this.guardsConsumer = guardsConsumer;
        this.interceptorsContextCreator = interceptorsContextCreator;
        this.interceptorsConsumer = interceptorsConsumer;
        this.applicationRef = applicationRef;
        this.handlerMetadataStorage = new handler_metadata_storage_1.HandlerMetadataStorage();
        this.contextUtils = new context_utils_1.ContextUtils();
        this.DEFAULT_RESPONSE_CODE = 200;
        this.responseController = new json_rpc_response_controller_1.JsonRpcResponseController(applicationRef);
    }
    create(instance, callback, methodName, module, contextId = constants_1.STATIC_CONTEXT, inquirerId) {
        const contextType = 'http';
        const { argsLength, fnHandleResponse, paramtypes, getParamsMetadata, responseHeaders, hasCustomHeaders, httpStatusCode, } = this.getMetadata(instance, callback, methodName, contextType);
        const paramsOptions = this.contextUtils.mergeParamsMetatypes(getParamsMetadata(module, contextId, inquirerId), paramtypes);
        const pipes = this.pipesContextCreator.create(instance, callback, module, contextId, inquirerId);
        const guards = this.guardsContextCreator.create(instance, callback, module, contextId, inquirerId);
        const interceptors = this.interceptorsContextCreator.create(instance, callback, module, contextId, inquirerId);
        const fnCanActivate = this.createGuardsFn(guards, instance, callback, contextType);
        const fnApplyPipes = this.createPipesFn(pipes, paramsOptions);
        const handler = (args, req, res, next) => () => __awaiter(this, void 0, void 0, function* () {
            if (fnApplyPipes) {
                yield fnApplyPipes(args, req, res, next);
            }
            return callback.apply(instance, args);
        });
        return (req, res, next) => __awaiter(this, void 0, void 0, function* () {
            const args = this.contextUtils.createNullArray(argsLength);
            if (fnCanActivate) {
                yield fnCanActivate([req, res, next]);
            }
            this.responseController.setStatus(res, httpStatusCode);
            if (hasCustomHeaders === true) {
                this.responseController.setHeaders(res, responseHeaders);
            }
            const result = yield this.interceptorsConsumer.intercept(interceptors, [req, res, next], instance, callback, handler(args, req, res, next), contextType);
            return yield fnHandleResponse(result, res);
        });
    }
    getMetadata(instance, callback, methodName, contextType) {
        const cacheMetadata = this.handlerMetadataStorage.get(instance, methodName);
        if (cacheMetadata) {
            return cacheMetadata;
        }
        const contextFactory = this.contextUtils.getContextFactory(contextType, instance, callback);
        const metadata = this.contextUtils.reflectCallbackMetadata(instance, methodName, constants_2.ROUTE_ARGS_METADATA) || {};
        const keys = Object.keys(metadata);
        const argsLength = this.contextUtils.getArgumentsLength(keys, metadata);
        const paramtypes = this.contextUtils.reflectCallbackParamtypes(instance, methodName);
        const getParamsMetadata = (moduleKey, contextId = constants_1.STATIC_CONTEXT, inquirerId) => this.exchangeKeysForValues(keys, metadata, moduleKey, contextId, inquirerId, contextFactory);
        const fnHandleResponse = this.createHandleResponseFn();
        const responseHeaders = this.reflectResponseHeaders(callback);
        const hasCustomHeaders = !(0, shared_utils_1.isEmpty)(responseHeaders);
        const handlerMetadata = {
            argsLength,
            fnHandleResponse,
            paramtypes,
            getParamsMetadata,
            httpStatusCode: this.DEFAULT_RESPONSE_CODE,
            hasCustomHeaders,
            responseHeaders,
        };
        this.handlerMetadataStorage.set(instance, methodName, handlerMetadata);
        return handlerMetadata;
    }
    exchangeKeysForValues(keys, metadata, moduleContext, contextId = constants_1.STATIC_CONTEXT, inquirerId, contextFactory = this.contextUtils.getContextFactory('http')) {
        this.pipesContextCreator.setModuleContext(moduleContext);
        return keys.map((key) => {
            const { index, data, pipes: pipesCollection } = metadata[key];
            const pipes = this.pipesContextCreator.createConcreteContext(pipesCollection, contextId, inquirerId);
            const type = this.contextUtils.mapParamType(key);
            if (key.includes(constants_2.CUSTOM_ROUTE_ARGS_METADATA)) {
                const { factory } = metadata[key];
                const customExtractValue = this.getCustomFactory(factory, data, contextFactory);
                return { index, extractValue: customExtractValue, type, data, pipes };
            }
            const numericType = Number(type);
            const extractValue = (req, res, next) => this.paramsFactory.exchangeKeyForValue(numericType, data, {
                req,
                res,
                next,
            });
            return { index, extractValue, type: numericType, data, pipes };
        });
    }
    getCustomFactory(factory, data, contextFactory) {
        return (0, shared_utils_1.isFunction)(factory)
            ? (...args) => factory(data, contextFactory(args))
            : () => null;
    }
    getParamValue(value, { metatype, type, data, }, pipes) {
        return __awaiter(this, void 0, void 0, function* () {
            if (type === route_paramtypes_enum_1.RouteParamtypes.BODY || (0, shared_utils_1.isString)(type)) {
                return this.pipesConsumer.apply(value, { metatype, type, data }, pipes);
            }
            return Promise.resolve(value);
        });
    }
    createGuardsFn(guards, instance, callback, contextType) {
        const canActivateFn = (args) => __awaiter(this, void 0, void 0, function* () {
            const canActivate = yield this.guardsConsumer.tryActivate(guards, args, instance, callback, contextType);
            if (!canActivate) {
                throw new common_1.ForbiddenException(constants_3.FORBIDDEN_MESSAGE);
            }
        });
        return guards.length ? canActivateFn : null;
    }
    createPipesFn(pipes, paramsOptions) {
        const pipesFn = (args, req, res, next) => __awaiter(this, void 0, void 0, function* () {
            yield Promise.all(paramsOptions.map((param) => __awaiter(this, void 0, void 0, function* () {
                const { index, extractValue, type, data, metatype, pipes: paramPipes, } = param;
                const value = extractValue(req, res, next);
                args[index] = yield this.getParamValue(value, { metatype, type, data }, pipes.concat(paramPipes));
            })));
        });
        return paramsOptions.length ? pipesFn : null;
    }
    createHandleResponseFn() {
        return (result, res) => __awaiter(this, void 0, void 0, function* () {
            result = yield this.transformToResult(result);
            return result;
        });
    }
    transformToResult(resultOrDeffered) {
        return __awaiter(this, void 0, void 0, function* () {
            if (resultOrDeffered && (0, shared_utils_1.isFunction)(resultOrDeffered.subscribe)) {
                return resultOrDeffered.toPromise();
            }
            return resultOrDeffered;
        });
    }
    reflectResponseHeaders(callback) {
        return Reflect.getMetadata(constants_2.HEADERS_METADATA, callback) || [];
    }
}
exports.JsonRpcContextCreator = JsonRpcContextCreator;
