"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.JsonRpcServer = void 0;
const common_1 = require("@nestjs/common");
const core_1 = require("@nestjs/core");
const lodash_1 = require("lodash");
const rxjs_1 = require("rxjs");
const shared_utils_1 = require("@nestjs/common/utils/shared.utils");
const operators_1 = require("rxjs/operators");
const json_rpc_exception_1 = require("./exception/json-rpc.exception");
const rpc_invalid_request_exception_1 = require("./exception/rpc-invalid-request.exception");
const rpc_method_not_found_exception_1 = require("./exception/rpc-method-not-found.exception");
let JsonRpcServer = class JsonRpcServer {
    constructor(httpAdapterHost) {
        this.httpAdapterHost = httpAdapterHost;
        this.needKeys = ['jsonrpc', 'method'];
        this.ignoreKeys = ['params', 'id'];
    }
    run(handlers, config) {
        this.handlers = handlers;
        this.httpAdapterHost.httpAdapter.post(config.path, this.onRequest.bind(this));
    }
    onRequest(request, response, next) {
        if (Array.isArray(request.body)) {
            this.batchRequest(request, response, next);
            return;
        }
        this.lifecycle(request, response, next).subscribe((result) => {
            this.sendResponse(response, request.body.id ? result : undefined);
        });
    }
    batchRequest(request, response, next) {
        const batch = request.body;
        if (batch.length === 0) {
            this.sendResponse(response, this.wrapRPCError({}, new rpc_invalid_request_exception_1.RpcInvalidRequestException()));
            return;
        }
        const requests = batch.map((body) => {
            return this.lifecycle(Object.assign(Object.assign({}, request), { body }), response, next);
        });
        (0, rxjs_1.forkJoin)(...requests).subscribe((results) => {
            const responses = results.filter((result) => {
                return result && result.id !== undefined;
            });
            this.sendResponse(response, responses.length === 0 ? undefined : responses);
        });
    }
    sendResponse(response, result) {
        this.httpAdapterHost.httpAdapter.setHeader(response, 'Content-Type', 'application/json');
        this.httpAdapterHost.httpAdapter.reply(response, JSON.stringify(result), common_1.HttpStatus.OK);
    }
    lifecycle(request, response, next) {
        return (0, rxjs_1.of)(request.body).pipe((0, operators_1.tap)((body) => this.assertRPCStructure(body)), (0, operators_1.tap)((body) => {
            if (this.handlers.has(body.method) === false) {
                throw new rpc_method_not_found_exception_1.RpcMethodNotFoundException();
            }
        }), (0, operators_1.switchMap)((body) => this.resolveWaitingResponse(body, request, response, next)), (0, operators_1.catchError)((err) => (0, rxjs_1.of)(err)), (0, operators_1.map)((result) => this.resolveResponseOrNullIfNotification(result, request.body)));
    }
    resolveResponseOrNullIfNotification(result, body) {
        if (result instanceof json_rpc_exception_1.RpcException === false && body.id) {
            return this.wrapRPCResponse(body, result);
        }
        if (result instanceof rpc_invalid_request_exception_1.RpcInvalidRequestException) {
            return this.wrapRPCError(body, result);
        }
        if (body.id === undefined) {
            return null;
        }
        return this.wrapRPCError(body, result);
    }
    resolveWaitingResponse(body, request, response, next) {
        const { method, id } = body;
        if (id === undefined) {
            this.handlers.get(method)(request, response, next);
            return (0, rxjs_1.of)(null);
        }
        const result = this.handlers.get(method)(request, response, next);
        if (result instanceof Promise) {
            return (0, rxjs_1.from)(result);
        }
        if (!this.isObservable(result)) {
            return (0, rxjs_1.of)(result);
        }
        return result;
    }
    isObservable(input) {
        return input && (0, shared_utils_1.isFunction)(input.subscribe);
    }
    wrapRPCResponse({ jsonrpc, id, method }, result = null) {
        return {
            jsonrpc,
            method,
            id,
            result,
        };
    }
    wrapRPCError({ jsonrpc = '2.0', method, id }, error) {
        return {
            jsonrpc,
            method,
            id,
            error,
        };
    }
    assertRPCStructure(body) {
        if (Array.isArray(body)) {
            for (const operation of body) {
                this.assertStructure(operation);
            }
        }
        else {
            this.assertStructure(body);
        }
        return body;
    }
    assertStructure(operation) {
        const keys = Object.keys(operation).filter((key) => {
            return this.ignoreKeys.includes(key) === false;
        });
        const isValidStructure = (0, lodash_1.isEqual)((0, lodash_1.sortBy)(this.needKeys), (0, lodash_1.sortBy)(keys)) &&
            this.isValidIdType(operation.id) &&
            typeof operation.method === 'string';
        if (isValidStructure) {
            return;
        }
        throw new rpc_invalid_request_exception_1.RpcInvalidRequestException();
    }
    isValidIdType(id) {
        const type = typeof id;
        if (type === 'undefined') {
            return true;
        }
        if (type === 'number' && Number.isInteger(id)) {
            return true;
        }
        return type === 'string' || id === null;
    }
};
exports.JsonRpcServer = JsonRpcServer;
exports.JsonRpcServer = JsonRpcServer = __decorate([
    (0, common_1.Injectable)(),
    __metadata("design:paramtypes", [core_1.HttpAdapterHost])
], JsonRpcServer);
