"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RpcCallbackProxy = void 0;
const constants_1 = require("@nestjs/core/injector/constants");
const execution_context_host_1 = require("@nestjs/core/helpers/execution-context-host");
const core_1 = require("@nestjs/core");
const request_constants_1 = require("@nestjs/core/router/request/request-constants");
const json_rpc_proxy_1 = require("./context/json-rpc-proxy");
const json_rpc_context_creator_1 = require("./context/json-rpc-context-creator");
const router_exception_filters_1 = require("@nestjs/core/router/router-exception-filters");
const route_params_factory_1 = require("@nestjs/core/router/route-params-factory");
const pipes_context_creator_1 = require("@nestjs/core/pipes/pipes-context-creator");
const pipes_consumer_1 = require("@nestjs/core/pipes/pipes-consumer");
const guards_context_creator_1 = require("@nestjs/core/guards/guards-context-creator");
const guards_consumer_1 = require("@nestjs/core/guards/guards-consumer");
const interceptors_context_creator_1 = require("@nestjs/core/interceptors/interceptors-context-creator");
const interceptors_consumer_1 = require("@nestjs/core/interceptors/interceptors-consumer");
class RpcCallbackProxy {
    constructor(config, container, injector) {
        this.config = config;
        this.container = container;
        this.injector = injector;
        this.routerProxy = new json_rpc_proxy_1.JsonRpcProxy();
        this.exceptionFiltersCache = new WeakMap();
        const httpAdapterRef = container.getHttpAdapterRef();
        this.executionContextCreator = new json_rpc_context_creator_1.JsonRpcContextCreator(new route_params_factory_1.RouteParamsFactory(), new pipes_context_creator_1.PipesContextCreator(container, this.config), new pipes_consumer_1.PipesConsumer(), new guards_context_creator_1.GuardsContextCreator(container, this.config), new guards_consumer_1.GuardsConsumer(), new interceptors_context_creator_1.InterceptorsContextCreator(container, this.config), new interceptors_consumer_1.InterceptorsConsumer(), httpAdapterRef);
        this.exceptionsFilter = new router_exception_filters_1.RouterExceptionFilters(container, this.config, httpAdapterRef);
    }
    create(rpcMethodHandler, moduleKey) {
        const { instanceWrapper, methodName, callback, } = rpcMethodHandler;
        const { instance } = instanceWrapper;
        const isRequestScoped = !instanceWrapper.isDependencyTreeStatic();
        return isRequestScoped
            ? this.createRequestScopedHandler(instanceWrapper, this.container.getModuleByKey(moduleKey), moduleKey, methodName)
            : this.createCallbackProxy(instance, callback, methodName, moduleKey);
    }
    createCallbackProxy(instance, callback, methodName, moduleRef, contextId = constants_1.STATIC_CONTEXT, inquirerId) {
        const executionContext = this.executionContextCreator.create(instance, callback, methodName, moduleRef, contextId, inquirerId);
        const exceptionFilter = this.exceptionsFilter.create(instance, callback, moduleRef, contextId, inquirerId);
        return this.routerProxy.createProxy(executionContext, exceptionFilter);
    }
    createRequestScopedHandler(instanceWrapper, moduleRef, moduleKey, methodName) {
        const { instance } = instanceWrapper;
        const collection = moduleRef.providers;
        return (req, res, next) => __awaiter(this, void 0, void 0, function* () {
            try {
                const contextId = this.getContextId(req);
                const contextInstance = yield this.injector.loadPerContext(instance, moduleRef, collection, contextId, instanceWrapper);
                return this.createCallbackProxy(contextInstance, contextInstance[methodName], methodName, moduleKey, contextId, instanceWrapper.id)(req, res, next);
            }
            catch (err) {
                let exceptionFilter = this.exceptionFiltersCache.get(instance[methodName]);
                if (!exceptionFilter) {
                    exceptionFilter = this.exceptionsFilter.create(instance, instance[methodName], moduleKey);
                    this.exceptionFiltersCache.set(instance[methodName], exceptionFilter);
                }
                const host = new execution_context_host_1.ExecutionContextHost([req, res, next]);
                exceptionFilter.next(err, host);
            }
        });
    }
    getContextId(request) {
        const contextId = core_1.ContextIdFactory.getByRequest(request);
        if (!request[request_constants_1.REQUEST_CONTEXT_ID]) {
            Object.defineProperty(request, request_constants_1.REQUEST_CONTEXT_ID, {
                value: contextId,
                enumerable: false,
                writable: false,
                configurable: false,
            });
            this.container.registerRequestProvider(request, contextId);
        }
        return contextId;
    }
}
exports.RpcCallbackProxy = RpcCallbackProxy;
