"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RpcRoutesResolver = exports.RPC_MAPPING_MESSAGE = void 0;
const constants_1 = require("@nestjs/common/constants");
const logger_service_1 = require("@nestjs/common/services/logger.service");
const rpc_callback_proxy_1 = require("./rpc-callback-proxy");
const RPC_MAPPING_MESSAGE = (name, path, method) => `Registered handler ${name} with method {${method}} on path {${path}}:`;
exports.RPC_MAPPING_MESSAGE = RPC_MAPPING_MESSAGE;
class RpcRoutesResolver {
    constructor(container, config, injector, rpcConfig, jsonRpcExplorer) {
        this.container = container;
        this.config = config;
        this.injector = injector;
        this.rpcConfig = rpcConfig;
        this.jsonRpcExplorer = jsonRpcExplorer;
        this.logger = new logger_service_1.Logger(RpcRoutesResolver.name);
        this.rpcCallbackProxy = new rpc_callback_proxy_1.RpcCallbackProxy(this.config, this.container, this.injector);
    }
    resolve(applicationRef, basePath) {
        const modules = this.container.getModules();
        const handlers = new Map();
        modules.forEach((module, moduleName) => {
            const { metatype } = module;
            let path = metatype ? this.getModulePathMetadata(metatype) : undefined;
            path = path ? basePath + path : basePath;
            const rpcMethodHandlers = this.jsonRpcExplorer.explore(module);
            const moduleRpcProxies = this.registerRouters(rpcMethodHandlers, moduleName, path);
            moduleRpcProxies.forEach(({ method, proxy }) => handlers.set(method, proxy));
        });
        return handlers;
    }
    registerRouters(routes, moduleKey, basePath) {
        const path = basePath + this.rpcConfig.path;
        return routes.map((rpcMethodHandler) => {
            const { instanceWrapper, rpcMethodName } = rpcMethodHandler;
            const { metatype } = instanceWrapper;
            const rpcHandlerName = metatype.name;
            this.logger.log((0, exports.RPC_MAPPING_MESSAGE)(rpcHandlerName, path, rpcMethodName));
            const proxy = this.rpcCallbackProxy.create(rpcMethodHandler, moduleKey);
            return { proxy, method: rpcMethodName };
        });
    }
    registerNotFoundHandler() { }
    registerExceptionHandler() { }
    getModulePathMetadata(metatype) {
        return Reflect.getMetadata(constants_1.MODULE_PATH, metatype);
    }
}
exports.RpcRoutesResolver = RpcRoutesResolver;
